/*
 * Copyright (c) 2005- Shinji Kashihara.
 * All rights reserved. This program are made available under
 * the terms of the Eclipse Public License v1.0 which accompanies
 * this distribution, and is available at epl-v10.html.
 */
package jp.sourceforge.mergedoc.pleiades.aspect;

import java.io.IOException;
import java.lang.instrument.IllegalClassFormatException;
import java.lang.reflect.Field;
import java.security.ProtectionDomain;

import javassist.CannotCompileException;
import javassist.CtClass;
import javassist.CtMethod;
import javassist.NotFoundException;
import jp.sourceforge.mergedoc.pleiades.log.Logger;

/**
 * ԌvO@\t LauncherTransformer łB
 * <p>
 * @author cypher256
 */
public class LauncherLoggingTransformer extends LauncherTransformer implements ITimingLogger {

	/** K[ */
	private static final Logger log = Logger.getLogger(LauncherLoggingTransformer.class);

	/** ̃NX */
	private static final String CLASS_NAME = LauncherLoggingTransformer.class.getName();

	/** Jn */
	private static final long startTime = System.currentTimeMillis();

	/** load \bh̏v */
	private volatile int loadTime;

	/** load \bȟĂяo */
	private volatile int loadCall;

	/** transform (ۃNX) \bh̏v */
	private volatile int transformAbstractTime;

	/** transform (ۃNX) \bȟĂяo */
	private volatile int transformAbstractCall;

	/** createCtClass (ۃNX) \bh̏v */
	private volatile int createCtClassTime;

	/** createCtClass (ۃNX) \bȟĂяo */
	private volatile int createCtClassCall;

	/** transform \bh̏v */
	private volatile int transformTime;

	/** transform \bȟĂяo */
	private volatile int transformCall;

	/** weaveUptimeMethod \bh̏v */
	private volatile int transformWorkbenchClassTime;

	/** weaveUptimeMethod \bȟĂяo */
	private volatile int transformWorkbenchClassCall;

	/** weaveStartTranslationAspect \bh̏v */
	private volatile int transformMainClassTime;

	/** weaveStartTranslationAspect \bȟĂяo */
	private volatile int transformMainClassCall;

	/**
	 * NgXtH[}[\z܂B
	 */
	public LauncherLoggingTransformer() {
		Pleiades.registTimingLogger(this);
	}

	/**
	 * eNX̓\bhɎԌvOo͂ǉ\bhłB
	 */
	@Override
	protected void load() {

		long startTime = System.currentTimeMillis();
		super.load();
		synchronized (this) {
			loadTime += System.currentTimeMillis() - startTime;
			loadCall++;
		}
	}

	/**
	 * eNX̓\bhɎԌvOo͂ǉ\bhłB
	 */
	@Override
	public byte[] transform(
			ClassLoader loader,
			String internalName,
			Class<?> classBeingRedefined,
			ProtectionDomain protectionDomain,
			byte[] bytecode) throws IllegalClassFormatException {

		long startTime = System.currentTimeMillis();
		byte[] result = super.transform(loader, internalName, classBeingRedefined, protectionDomain, bytecode);
		synchronized (this) {
			transformAbstractTime += System.currentTimeMillis() - startTime;
			transformAbstractCall++;
		}
		return result;
	}

	/**
	 * oCgR[hϊs܂B
	 */
	@Override
	protected byte[] transform(
			ClassLoader loader,
			String className,
			ProtectionDomain protectionDomain,
			byte[] bytecode)
		throws CannotCompileException, NotFoundException, IOException {

		long startTime = System.currentTimeMillis();
		byte[] result = super.transform(loader, className, protectionDomain, bytecode);
		synchronized (this) {
			transformTime += System.currentTimeMillis() - startTime;
			transformCall++;
		}
		return result;
	}

	/**
	 * eNX̓\bhɎԌvOo͂ǉ\bhłB
	 */
	@Override
	protected byte[] transformMainClass(ProtectionDomain protectionDomain, byte[] bytecode)
		throws CannotCompileException, NotFoundException, IOException {

		long startTime = System.currentTimeMillis();
		byte[] result = super.transformMainClass(protectionDomain, bytecode);
		synchronized (this) {
			transformMainClassTime += System.currentTimeMillis() - startTime;
			transformMainClassCall++;
		}
		return result;
	}

	/**
	 * Workbench NXϊ܂B
	 * eNX̏ɉAI̎Ԍvʂ̃Oo͏ǉ܂B
	 */
	@Override
	protected byte[] transformWorkbenchClass(ProtectionDomain protectionDomain, byte[] bytecode)
		throws CannotCompileException, NotFoundException, IOException {

		long startTime = System.currentTimeMillis();

		CtClass ctClass = createCtClass(bytecode, protectionDomain);
		CtMethod runStartupWithProgress =
			ctClass.getMethod("runStartupWithProgress", "(ILjava/lang/Runnable;)V");
		runStartupWithProgress.insertAfter(CLASS_NAME + ".logUpTime();");
		byte[] result = ctClass.toBytecode();
		
		synchronized (this) {
			transformWorkbenchClassTime += System.currentTimeMillis() - startTime;
			transformWorkbenchClassCall++;
		}
		return result;
	}

	/**
	 * eNX̓\bhɎԌvOo͂ǉ\bhłB
	 */
	@Override
	protected CtClass createCtClass(
			byte[] bytecode,
			ProtectionDomain protectionDomain) throws IOException, NotFoundException {

		long startTime = System.currentTimeMillis();
		CtClass result = super.createCtClass(bytecode, protectionDomain);
		synchronized (this) {
			createCtClassTime += System.currentTimeMillis() - startTime;
			createCtClassCall++;
		}
		return result;
	}

	/**
	 * gXtH[}[j܂B
	 */
	public static void destroy() {

		// I̎Ԍv
		logLapTime();

		// eNX̏ (Fstatic)
		LauncherTransformer.destroy();
	}

	/**
	 * ׂĂ̓o^ς݃gXtH[}[̋NԌvOo͂܂B
	 */
	public static void logUpTime() {

		// ԌvOo
		logLapTime();

		// NԂ̏o
		// Eclipse Jnł͂ȂAPleiades Jnv
		//long startTime = Long.valueOf(System.getProperty("eclipse.startTime"));
		long curTime = System.currentTimeMillis();
		double time = (curTime - startTime) / 1000d;
		log.debug(String.format(
				"Eclipse ̋N܂BN: %.3f b", time));
	}

	/**
	 * ׂĂ̓o^ς݃gXtH[}[̎ԌvOo͂܂B
	 */
	public static void logLapTime() {

		long summuryTime = 0;
		for (ITimingLogger logger : Pleiades.getTimeingLoggers()) {

			// vȌo
			summuryTime += logger.logTime();

			// vlZbg
			logger.reset();
		}
		log.debug(String.format("Pleiades Ԍv %6.3f b", summuryTime / 1000d));
	}

	/**
	 * ԌvOo͂܂B
	 * @return vԍv (b)
	 */
	public long logTime() {

		String msg = "%-31s %8d ďov %6.3f b";

		log.debug(String.format(msg, "load",
				loadCall, loadTime / 1000d));
		log.debug(String.format(msg, "transform (Abstract)",
				transformAbstractCall, transformAbstractTime / 1000d));
		log.debug(String.format(msg, "  transform",
				transformCall, transformTime / 1000d));
		log.debug(String.format(msg, "    createCtClass (Abstract)",
				createCtClassCall, createCtClassTime / 1000d));
		log.debug(String.format(msg, "    transformMainClass",
				transformMainClassCall, transformMainClassTime / 1000d));
		log.debug(String.format(msg, "    transformWorkbenchClass",
				transformWorkbenchClassCall, transformWorkbenchClassTime / 1000d));

		long summuryTime = transformAbstractTime + transformTime + createCtClassTime +
			transformMainClassTime + transformWorkbenchClassTime;

		log.debug(String.format("NXv %6.3f b", summuryTime / 1000d));

		return summuryTime;
	}

	/**
	 * vlZbg܂B
	 */
	public void reset() {

		synchronized (this) {
			Field[] fields = getClass().getDeclaredFields();
			for (int i = 0; i < fields.length; i++) {
				Field field = fields[i];
				if (field.getType() == Integer.TYPE) {
					try {
						field.setAccessible(true);
						field.set(this, 0);
					} catch (Exception e) {
						log.error("tB[h̏Ɏs܂B", e);
					}
				}
			}
		}
	}
}
