/*
 * Copyright (c) 2005- Shinji Kashihara.
 * All rights reserved. This program are made available under
 * the terms of the Eclipse Public License v1.0 which accompanies
 * this distribution, and is available at epl-v10.html.
 */
package jp.sourceforge.mergedoc.pleiades.log;

import java.io.PrintStream;
import java.lang.reflect.Constructor;
import java.util.Date;

/**
 * Log4J CNȃC^tF[XۃK[łB
 * <p>
 * @author cypher256
 */
public abstract class Logger {

	/** K[ENXVXeEvpeB[̃L[ */
	public static final String LOGGER_CLASS_NAME = "pleiades.logger.class.name";

	/** OExVXeEvpeB[̃L[ */
	public static final String LOG_LEVEL = "pleiades.log.level";

	/** OEx񋓌^ */
	public static enum Level {
		/** fobO */
		DEBUG,
		/**  */
		INFO,
		/** x */
		WARN,
		/** G[ */
		ERROR,
		/** vI */
		FATAL
	}

	/** ŏo̓OEx */
	private static volatile Level minLevel;

	/** ̃K[𐶐NX */
	private final String className;

	/**
	 * K[擾܂B
	 * <p>
	 * @param clazz NX
	 * @return K[
	 */
	public static Logger getLogger(Class<?> clazz) {

		synchronized (Logger.class) {

			if (minLevel == null) {
				String logLevel = System.getProperty(LOG_LEVEL, Level.INFO.toString());
				try {
					minLevel = Level.valueOf(logLevel.toUpperCase());
				} catch (IllegalStateException e) {
					throw new IllegalStateException(
							"VXevpeB " + LOG_LEVEL + "=" + logLevel +
							" Ɏw肷l " + Level.class.getName() +
							"Œ`ꂽ^łKv܂B");
				}
				System.out.println(LOG_LEVEL + "=" + logLevel);
			}
		}

		String loggerClassName =
			System.getProperty(LOGGER_CLASS_NAME, SystemOutLogger.class.getName());
		try {

			Constructor<?> cons = Class.forName(loggerClassName).getDeclaredConstructor(Class.class);
			cons.setAccessible(true);
			return (Logger) cons.newInstance(clazz);

		} catch (Exception e) {
			throw new IllegalStateException(
					"VXevpeB " + LOGGER_CLASS_NAME + "=" +
					loggerClassName +
					" Ɏw肳ꂽK[NX[hł܂łB", e);
		}
	}

	/**
	 * K[\z܂B
	 * <p>
	 * @param clazz NX
	 */
	protected Logger(Class<?> clazz) {
		this.className = clazz.getName().replaceFirst("(\\w+\\.)+", "");
	}

	/**
	 * o͐ƂȂ PrintStream 擾܂B
	 * <p>
	 * @return o͐ƂȂ PrintStream
	 */
	abstract protected PrintStream getOut();

	/**
	 * bZ[Wo͂܂B
	 * <p>
	 * @param level OEx
	 * @param message bZ[W
	 * @param t OBnull ̏ꍇ message ݂̂óB
	 */
	protected void println(Level level, String message, Throwable t) {

		Date date = new Date();
		String threadName = Thread.currentThread().getName();
		String s = String.format(
				"%-5s %tT.%tL [%s] (%s) %s",
				level, date, date, threadName, className, message);

		PrintStream out = getOut();
		out.println(s);
		if (t != null) {
			t.printStackTrace(out);
		}
	}

	/**
	 * fobOEOo͂܂B
	 * <p>
	 * @param message bZ[W
	 */
	public void debug(String message) {
		debug(message, null);
	}

	/**
	 * fobOEOo͂܂B
	 * <p>
	 * @param message bZ[W
	 * @param e O
	 */
	public void debug(String message, Throwable e) {
		if (isDebugEnabled()) println(Level.DEBUG, message, e);
	}

	/**
	 * 񃍃Oo͂܂B
	 * <p>
	 * @param message bZ[W
	 */
	public void info(String message) {
		info(message, null);
	}

	/**
	 * 񃍃Oo͂܂B
	 * <p>
	 * @param message bZ[W
	 * @param e O
	 */
	public void info(String message, Throwable e) {
		if (isInfoEnabled()) println(Level.INFO, message, e);
	}

	/**
	 * xOo͂܂B
	 * <p>
	 * @param message bZ[W
	 */
	public void warn(String message) {
		warn(message, null);
	}

	/**
	 * xOo͂܂B
	 * <p>
	 * @param message bZ[W
	 * @param e O
	 */
	public void warn(String message, Throwable e) {
		if (isWarnEnabled()) println(Level.WARN, message, e);
	}

	/**
	 * G[EOo͂܂B
	 * <p>
	 * @param message bZ[W
	 */
	public void error(String message) {
		error(message, null);
	}

	/**
	 * G[EOo͂܂B
	 * <p>
	 * @param message bZ[W
	 * @param e O
	 */
	public void error(String message, Throwable e) {
		if (isErrorEnabled()) println(Level.ERROR, message, e);
	}

	/**
	 * vIG[EOo͂܂B
	 * <p>
	 * @param message bZ[W
	 */
	public void fatal(String message) {
		fatal(message, null);
	}

	/**
	 * vIG[EOo͂܂B
	 * <p>
	 * @param message bZ[W
	 * @param e O
	 */
	public void fatal(String message, Throwable e) {
		if (isFatalEnabled()) println(Level.FATAL, message, e);
	}

	/**
	 * fobOEOL肵܂B
	 * <p>
	 * @return Lȏꍇ true
	 */
	public boolean isDebugEnabled() {
		return	minLevel == Level.DEBUG;
	}

	/**
	 * 񃍃OL肵܂B
	 * <p>
	 * @return Lȏꍇ true
	 */
	public boolean isInfoEnabled() {
		return	minLevel == Level.DEBUG || minLevel == Level.INFO ;
	}

	/**
	 * xOL肵܂B
	 * <p>
	 * @return Lȏꍇ true
	 */
	public boolean isWarnEnabled() {
		return	minLevel == Level.DEBUG || minLevel == Level.INFO  ||
				minLevel == Level.WARN ;
	}

	/**
	 * G[EOL肵܂B
	 * <p>
	 * @return Lȏꍇ true
	 */
	public boolean isErrorEnabled() {
		return	minLevel == Level.DEBUG || minLevel == Level.INFO  ||
				minLevel == Level.WARN  || minLevel == Level.ERROR;
	}

	/**
	 * vIG[EOL肵܂B
	 * <p>
	 * @return Lȏꍇ true
	 */
	public boolean isFatalEnabled() {
		return true;
	}
}
